#ifndef __XMLIO_DURATION__
#define __XMLIO_DURATION__

namespace XMLIOSERVER
{
   class AbstractCalendar;

   typedef long long int Time;

   typedef struct _duration
   {
      public :

         friend std::ostream& operator<<(std::ostream& out, const struct _duration& d)
         {
            bool testValue = true;
            if(d.year   != 0.0) { testValue = false; out << d.year   << "y " ; }
            if(d.month  != 0.0) { testValue = false; out << d.month  << "mo "; }
            if(d.day    != 0.0) { testValue = false; out << d.day    << "d " ; }
            if(d.hour   != 0.0) { testValue = false; out << d.hour   << "h " ; }
            if(d.minute != 0.0) { testValue = false; out << d.minute << "mi "; }
            if(d.second != 0.0 || testValue) { out << d.second << "s " ; }
            return (out);
         }

         friend std::istream& operator>>(std::istream& in, struct _duration& d)
         {
            double v = 1.0;
            char   c = '/';
            while (!in.eof())
            {
               in >> v >> c;
               switch (c)
               {
                  case 'y': d.year   = v; continue;
                  case 'd': d.day    = v; continue;
                  case 'h': d.hour   = v; continue;
                  case 's': d.second = v; continue;
                  case 'm':
                     in >> c;
                     if (c == 'i') d.minute   = v;
                     else if(c == 'o') d.month   = v;
                     else
                     {
                        string valc("m"); valc.append(1, c);
                        WARNING("La chaine \""+valc+"\" ne permet pas de définir une unité de durée.");
                        continue;
                     }
                     continue;
                  default:
                     string valc; valc.append(1, c);
                     WARNING("La chaine \""+valc+"\" ne permet pas de définir une unité de durée.");
                     continue;
               }
            }
            return (in);
         }

         static struct _duration FromString(const string& str)
         {
            struct _duration dr = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
            std::istringstream iss(str);
            iss >> dr; return dr;
         }

         struct _duration& resolve(const AbstractCalendar&);

         double year, month, day, hour, minute, second;

   } Duration;

   // Les principales durées constantes : année, mois, jour, ...
   static const Duration Year   = {1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
                         Month  = {0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
                         Week   = {0.0, 0.0, 7.0, 0.0, 0.0, 0.0},
                         Day    = {0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
                         Hour   = {0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
                         Minute = {0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
                         Second = {0.0, 0.0, 0.0, 0.0, 0.0, 1.0};

} // namespace XMLIOSERVER

#endif // __XMLIO_DURATION__
